from abc import ABCMeta, abstractmethod
#action manager abstract class to be implemented by all 
# action managers
class ActionManager(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def validate(self):
        pass

    @abstractmethod
    def execute(self):
        pass



#!/usr/bin/env python

from ansible import errors
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser
import StringIO


def get_host_ips(topo):
    host_public_ips = []
    for group in topo['os_server_res']:
        host_public_ips.append(str(group['server']['public_v4']))
    return host_public_ips


def get_layout_hosts(inv):
    return inv['hosts'].keys()


def get_layout_host_groups(inv):
    return inv['host_groups'].keys()


def add_sections(config, section_list):
    for section in section_list:
        config.add_section(section)
    return config


def set_children(config, inv):
    for host_group in inv['host_groups']:
        if "children" in inv['host_groups'][host_group]:
            config.add_section(host_group+":"+"children")
            for child in inv['host_groups'][host_group]['children']:
                config.set(host_group+":"+"children", child)
    return config


def set_vars(config, inv):
    for host_group in inv['host_groups']:
        if "vars" in inv['host_groups'][host_group]:
            config.add_section(host_group+":"+"vars")
            for var in inv['host_groups'][host_group]['vars']:
                config.set(host_group + ":" + "vars",
                           var,
                           inv['host_groups'][host_group]['vars'][var])
    return config


def add_ips_to_groups(config, inven_hosts, layout):
    # create a ip to host mapping based on count
    ip_to_host = {}
    for host_name in layout['hosts']:
        count = layout['hosts'][host_name]['count']
        host_list = []
        for i in range(0, count):
            item = inven_hosts.pop()
            host_list.append(item)
        ip_to_host[host_name] = host_list
    # add ips to the host groups in inventory
    for host_name in layout['hosts']:
        host_ips = ip_to_host[host_name]
        for ip in host_ips:
            for host_group in layout['hosts'][host_name]['host_groups']:
                config.set(host_group, ip)
    return config


def add_common_vars(config, host_groups, layout):
    common_vars = layout['vars']
    for group in host_groups:
        items = dict(config.items(group)).keys()
        config.remove_section(group)
        config.add_section(group)
        for item in items:
            host_string = item
            for var in common_vars:
                if common_vars[var] == "__IP__":
                    host_string += " " + var + "=" + item + " "
            config.set(group, host_string)
    return config


def openstack_inventory(topo, layout):
    inventory = ConfigParser(allow_no_value=True)
    no_of_groups = len(topo['os_server_res'])
    layout_hosts = get_layout_hosts(layout)
    inven_hosts = get_host_ips(topo)
    # adding sections to respective host groups
    host_groups = get_layout_host_groups(layout)
    inventory = add_sections(inventory, host_groups)
    # set children for each host group
    inventory = set_children(inventory, layout)
    # set vars for each host group
    inventory = set_vars(inventory, layout)
    # add ip addresses to each host
    inventory = add_ips_to_groups(inventory, inven_hosts, layout)
    inventory = add_common_vars(inventory, host_groups, layout)
    output = StringIO.StringIO()
    inventory.write(output)
    return output.getvalue()


class FilterModule(object):
    ''' A filter to fix interface's name format '''
    def filters(self):
        return {
            'openstack_inventory': openstack_inventory
        }



#!/usr/bin/env python

import os
import logging

try:
    import configparser as ConfigParser
except ImportError:
    import ConfigParser as ConfigParser

from linchpin.exceptions import LinchpinError
from linchpin.version import __version__


class LinchpinContext(object):

    """
    LinchpinContext object, which will be used to manage the cli,
    and load the configuration file.
    """


    def __init__(self):
        """
        Initializes basic variables
        """

        self.version = __version__
        self.verbose = False

        lib_path = '{0}'.format(os.path.dirname(
            os.path.realpath(__file__))).rstrip('/')
        self.lib_path = os.path.realpath(os.path.join(lib_path, os.pardir))

        self.workspace = os.path.realpath(os.path.curdir)


    def load_config(self, lpconfig=None):
        """
        Create self.cfgs from the linchpin configuration file.

        These are the only hardcoded values, which are used to find
        the config file. The linchpin.conf file is found
        at `/linchpin/library/path/linchpin.conf`.

        Alternatively, a full path to the linchpin configuration file
        can be passed.

        :param lpconfig: absolute path to a linchpin config (default: None)

        """

        self.cfgs = {}

        expanded_path = None
        config_found = False

        if lpconfig:
            CONFIG_PATH = [ lpconfig ]
        else:
            # simply modify this variable to adjust where linchpin.conf can be found
            CONFIG_PATH = [
                '{0}/linchpin.conf'.format(self.lib_path)
            ]

        for path in CONFIG_PATH:
            expanded_path = (
                "{0}".format(os.path.realpath(os.path.expanduser(path))))

            # implement first found
            if os.path.exists(expanded_path):
                # logging before the config file is setup doesn't work
                # if messages are needed before this, use print.
                config_found = True
                break

        if not config_found:
            raise LinchpinError('Configuration file not found in'
                                ' path: {0}'.format(CONFIG_PATH))

        config = ConfigParser.SafeConfigParser()
        try:
            f = open(path)
            config.readfp(f)
            f.close()
        except ConfigParser.InterpolationSyntaxError as e:
            raise LinchpinError('Unable to parse configuration file properly:'
                    ' {0}'.format(e))

        for section in config.sections():
            if section not in self.cfgs:
                self.cfgs[section] = {}

            # add evars to the ansible extra_vars when running a playbook.
            for k, v in config.items(section):
                if section != 'evars':
                    self.cfgs[section][k] = v
                else:
                    try:
                        self.cfgs[section][k] = config.getboolean(section, k)
                    except ValueError as e:
                        self.cfgs[section][k] = v


    def load_global_evars(self):

        """
        Instantiate the evars variable, then load the variables from the
        'evars' section in linchpin.conf. This will then be passed to
        invoke_linchpin, which passes them to the Ansible playbook as needed.

        """

        self.evars = self.cfgs.get('evars', {})


    def get_cfg(self, section=None, key=None, default=None):
        """
        Get cfgs value(s) by section and/or key, or the whole cfgs object

        :param section: section from ini-style config file

        :param key: key to get from config file, within section

        :param default: default value to return if nothing is found.
        Does not apply if section is not provided.
        """

        if section:
            s = self.cfgs.get(section, default)
            if key and s:
                return self.cfgs[section].get(key, default)
            return s
        return self.cfgs


    def set_cfg(self, section, key, value):
        """
        Set a value in cfgs. Does not persist into a file,
        only during the current execution.

        :param section: section within ini-style config file
        :param key: key to use
        :param value: value to set into section within config file
        """

        if not self.cfgs.get(section):
            self.cfgs.update({section: {}})

        self.cfgs[section][key] = value


    def get_evar(self, key=None, default=None):
        """
        Get the current evars (extra_vars)

        :param key: key to use

        :param default: default value to return if nothing is found (default: None)
        """

        if key:
            return self.evars.get(key, default)

        return self.evars


    def set_evar(self, key, value):
        """
        Set a value into evars (extra_vars). Does not persist into a file,
        only during the current execution.

        :param key: key to use

        :param value: value to set into evars
        """

        self.set_cfg('evars', key, value)


    def setup_logging(self):

        """
        Setup logging to the console only

        .. attention:: Please implement this function in a subclass

        """

        self.console = logging.getLogger('lp_console')
        self.console.setLevel(logging.INFO)

        ch = logging.StreamHandler()
        ch.setLevel(logging.INFO)

        formatter = logging.Formatter('%(levelname)s %(asctime)s %(message)s')
        ch.setFormatter(formatter)

        self.console.addHandler(ch)


    def log(self, msg, **kwargs):
        """
        Logs a message to a logfile

        :param msg: message to output to log

        :param level: keyword argument defining the log level

        """

        lvl = kwargs.get('level')

        if lvl is None:
            lvl = logging.INFO

        self.console.log(logging.INFO, msg)


    def log_info(self, msg):
        """Logs an INFO message"""

        self.log(msg, level=logging.INFO)


    def log_debug(self, msg):
        """Logs a DEBUG message"""

        self.log(msg, level=logging.DEBUG)


    def log_state(self, msg):
        """
        Logs nothing, just calls pass

        .. attention:: state messages need to be implemented in a subclass
        """

        pass



#!/usr/bin/env python
import os
import sys
import abc
import StringIO
from ansible import errors

def format_networks(networks):
    #"net-name=atomic-e2e-jenkins-test,net-name=atomic-e2e-jenkins-test2"
    nics = ["net-name="+net for net in networks]
    nics = ",".join(nics)
    return str(nics)

class FilterModule(object):
    ''' A filter to fix network format '''
    def filters(self):
        return {
            'os_net': format_networks
        }



import requests
import json


class GitHub(object):

    def __init__(self, url):
        try:
            self.url = url.strip(".git").strip("/")
            self.owner = self.url.split("/")[-2]
            self.repo = self.url.split("/")[-1].strip("/")
            self.api_url = "http://api.github.com/repos"
        except Exception as e:
            raise  Exception("Incorrect URL")  

    def get_rq_url(self, directory):
        return self.api_url+"/"+self.owner+"/"+self.repo+"/contents/"+directory

    def list_files(self, directory):
        rq_url = self.get_rq_url(directory)
        rq = requests.get(rq_url)
        if (rq.ok):
            files = json.loads(rq.text or rq.content)
            return files

    def download_file(self, dest, directory, file_name):
        pass



#!/usr/bin/env python

import abc
import StringIO
from ansible import errors

try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser

from InventoryFilter import InventoryFilter


class DuffyInventory(InventoryFilter):

    def get_hostnames(self, topo):
        hostnames = []
        for group in topo['duffy_res']:
            for host in group['hosts']:
                hostnames.append(host)
        return hostnames

    def get_host_ips(self, topo):
        return self.get_hostnames(topo)


    def get_inventory(self, topo, layout):
        if len(topo['duffy_res']) == 0:
            return ""
        inventory = ConfigParser(allow_no_value=True)
        layout_hosts = self.get_layout_hosts(layout)
        inven_hosts = self.get_hostnames(topo)
        # adding sections to respective host groups
        host_groups = self.get_layout_host_groups(layout)
        self.add_sections(host_groups)
        # set children for each host group
        self.set_children(layout)
        # set vars for each host group
        self.set_vars(layout)
        # add ip addresses to each host
        self.add_ips_to_groups(inven_hosts, layout)
        self.add_common_vars(host_groups, layout)
        output = StringIO.StringIO()
        self.config.write(output)
        return output.getvalue()



#!/usr/bin/env python

import os
import sys
import shutil


####
#A cli utility module for performing tasks related to the linchpin cli
####

#def find_topology(ctx, topology):
#
#    print("Not found")
#    print("Finding topology in package ")
#    topos = self.lp_topo_list()
#    topos = [x["name"] for x in topos]
#    if topology in topos:
#        print("Found!! copying it to local folder")
#        self.lp_topo_get(topology)
#        return os.path.abspath("./topologies/"+topology)
#
#    print("Not found")
#    print("Finding topology from upstream")
#    topos = self.lp_topo_list(pf["topology_upstream"])
#    topos = [x["name"] for x in topos]
#    if topology in topos:
#        print("Found!!")
#        print("Fetching topology from upstream !")
#        self.lp_topo_get(topology, pf["topology_upstream"])
#        return os.path.abspath("./topologies/"+topology)
#    print("Not Found !!")


#    @staticmethod
#    def mkdir(dir_path):
#        if os.path.exists(dir_path):
#            shutil.rmtree(dir_path)
#        os.makedirs(dir_path)
#
#
#    @staticmethod
#    def display(message, m_type="print"):
#        if m_type == "print":
#            click.echo(message+":"+MSGS[message])
#        if m_type == "prompt":
#            while True:
#                reply = raw_input(message+":"+MSGS[message])
#                if (reply.lower() == "y" or reply.lower() == "yes"):
#                    return True
#                elif (reply.lower() == "n" or reply.lower() == "no"):
#                    sys.exit(0)
#        sys.exit(0)
#
#
#    @staticmethod
#    def list_by_ext(dir_path, ext):
#        files = []
#        for file in os.listdir(dir_path):
#            if file.endswith(ext):
#                files.append(dir_path+"/"+file)
#        return files
#
#
#    @staticmethod
#    def list_files(path):
#        files = os.listdir(path)
#        counter = 1
#        all_files = []
#        for filename in files:
#            file_tuple = [counter, filename]
#            all_files.append(file_tuple)
#            counter += 1
#        return all_files
#
#
#    @staticmethod
#    def get_file(src, dest):
#        try:
#            fd = open(src, "r")
#            name = fd.name.split("/")[-1]
#            inp = fd.read()
#            open(dest+name, "w").write(inp)
#            fd.close()
#        except Exception as e:
#            click.echo("get file aborted !!!")
#            click.echo(str(e))
#
#
#    @staticmethod
#    def copy_files(src, dest, dir_list):
#        for direc in dir_list:
#            dest_dir = dest + direc + "/"
#            src_dir = src + direc + "/"
#            for file in os.listdir(src_dir):
#                src_path = src_dir+file
#                shutil.copy(src_path, dest_dir)
#
#
#    @staticmethod
#    def checkpaths(workspace=None):
#        """ checks whether the linchpin layout already exists in cwd"""
#        if workspace is None:
#            cur_dir = os.getcwd()
#        else:
#            cur_dir = os.getcwd()
#        layout_files = ['layouts', 'topologies', 'PinFile']
#        for f in layout_files:
#            if f in os.listdir(cur_dir):
#                return True
#
#
#    @staticmethod
#    def search_path(name, path):
#        """ searches for files by name in a given path """
#        for root, dirs, files in os.walk(path):
#            if name in files:
#                return os.path.join(root, name)
#
#
#    @staticmethod
#    def tabulate_print(items, headers):
#        print_items = []
#        for i in range(0, len(items)):
#            print_items.append((i+1, items[i]["name"]))
#        print tabulate(print_items, headers, tablefmt="fancy_grid")
#
#
#    @staticmethod
#    def write_to_file(dest_path, output):
#        if os.path.isfile(dest_path):
#            click.echo("File exists at path "+dest_path)
#            ans = click.prompt('Do you want to overwrite it ?', default='Y')
#            if ans.lower() in ['no','n']:
#                pass
#            elif ans.lower() in ['yes','y']:
#                with open(dest_path, "w") as txt:
#                    txt.write(output)
#                click.echo("File created successfully")
#        else:
#            with open(dest_path, "w") as txt:
#                txt.write(output)
#            click.echo("File created successfully")
#
#
#    @staticmethod
#    def touch(fname):
#        if os.path.exists(fname):
#            os.utime(fname, None)
#        else:
#            open(fname, 'a').close()
#
#
#
#
#
#    def get_file(src, dest, link=False):
#        if link:
#            r = requests.get(src)
#            name = src.split("/")[-1]
#            open(dest+"/"+name, "w").write(r.content)
#        else:
#            fd = open(src, "r")
#            name = fd.name.split("/")[-1]
#            inp = fd.read()
#            open(dest+name, "w").write(inp)
#            fd.close()
#
#
#    def list_files(path):
#        files = os.listdir(path)
#        all_files = []
#        for filename in files:
#            file_dict = {"name": filename}
#            all_files.append(file_dict)
#        return all_files
#
#


#    def list_by_ext(dir_path, ext):
#        files = []
#        for file in os.listdir(dir_path):
#            if file.endswith(ext):
#                files.append(dir_path+"/"+file)
#        return files
#
#
#    def list_files(path):
#        files = os.listdir(path)
#        counter = 1
#        all_files = []
#        for filename in files:
#            file_tuple = [counter, filename]
#            all_files.append(file_tuple)
#            counter += 1
#        return all_files
#
#
#    def get_file(src, dest):
#        try:
#            fd = open(src, "r")
#            name = fd.name.split("/")[-1]
#            inp = fd.read()
#            open(dest+name, "w").write(inp)
#            fd.close()
#        except Exception as e:
#            click.echo("get file aborted !!!")
#            click.echo(str(e))
#
#
#    def copy_files(path, dir_list, config):
#        for direc in dir_list:
#            dest = path+"/"+direc+"/"
#            src = config.clipath+"/templates/"+direc+"/"
#            for filename in os.listdir(src):
#                src_file = src+filename
#                shutil.copy(src_file, dest)
#
#
#    def checkpaths():
#        """ checks whether the linchpin layout already exists in cwd"""
#        cur_dir = os.getcwd()
#        # print os.listdir(cur_dir)
#        layout_files = ['layouts', 'topologies', 'PinFile']
#        for f in layout_files:
#            if f in os.listdir(cur_dir):
#                return True
#
#
#    def parse_yaml(pf):
#        """ parses yaml file into json object """
#        with open(pf, 'r') as stream:
#            try:
#                pf = yaml.load(stream)
#                return pf
#            except yaml.YAMLError as exc:
#                print(exc)
#
#
#    def search_path(name, path):
#        """ searches for files by name in a given path """
#        for root, dirs, files in os.walk(path):
#            if name in files:
#                return os.path.join(root, name)
#



class LinchpinError(Exception):

    def __init__(self,*args,**kwargs):
        Exception.__init__(self,*args,**kwargs)

class HookError(LinchpinError):
    
    def __init__(self,*args,**kwargs):
        LinchpinError.__init__(self,*args,**kwargs)

class StateError(LinchpinError):
    
    def __init__(self,*args,**kwargs):
        LinchpinError.__init__(self,*args,**kwargs)

class ActionManagerError(LinchpinError):
    def __init__(self,*args,**kwargs):
        LinchpinError.__init__(self,*args,**kwargs)



import os
import ast
from setuptools import setup, find_packages
from pip.req import parse_requirements
import os

with open('linchpin/version.py') as f:
    for line in f:
        if line.startswith('__version__'):
            ver = ast.parse(line).body[0].value.s
            break

# reading requirements from requirements.txt
dir_path = os.path.dirname(os.path.realpath(__file__))
reqs_file = 'requirements.txt'.format(dir_path)
with open(reqs_file) as f:
    required = f.read().splitlines()

ignore_dir = ['.git']

setup(
    name='linchpin',
    version=ver,
    description = 'Ansible based multi cloud orchestrator',
    author = 'samvaran kashyap rallabandi',
    author_email = 'linchpin@redhat.com',
    url = 'http://linchpin.readthedocs.io/',
    setup_requires=required,
    install_requires=required,
    entry_points='''
        [console_scripts]
        linchpin=linchpin:runcli
    ''',
    extras_require = {
        'krbV': ["python-krbV"],
        'beaker':  ['beaker-client>=23.3'],
        'docs': ["docutils","sphinx","sphinx_rtd_theme"],
        'tests': ["nose","mock","coverage"],
        'libvirt': ["libvirt-python>=3.0.0", "lxml"],
    },
#    dependency_links = ['https://github.com/eevee/camel/archive/v0.1.tar.gz#egg=camel-99.1'],
    zip_safe=False,
    packages=find_packages(),
    include_package_data=True,
    scripts=['scripts/linchpin_complete.sh']
)
